using System;
using Structure.Graph2D.IO;

namespace Structure.Graph2D.GML
{
	public class Writer: WriterBase
	{
		private System.IO.TextWriter _writer;
		
		public Writer(string fileName):
			this(new System.IO.FileStream(fileName, System.IO.FileMode.Create, System.IO.FileAccess.Write))
		{
		}
		
		public Writer(System.IO.Stream stream)
		{
			_writer = new System.IO.StreamWriter(stream, System.Text.Encoding.ASCII);
			_writer.WriteLine("graph");
			_writer.WriteLine("[");
			_writer.WriteLine("directed 1");
		}
		
		private static string Encode(string value)
		{
			string result = "";
			value = value.Replace("|+", "\n").Replace("|", "\n");
			for (int i = 0; i < value.Length; i++)
			{
				Char ch = value[i];
				int code = System.Convert.ToInt32(ch);
				if ((code >= 128) || (ch == '&') || (ch == '"'))
				{
					result += String.Format("&#x{0};", code.ToString("x"));
				}
				else
				{
					result += ch;
				}
			}
			return result;
		}
		
		#region IWriter interface
		public override void Write(Edge edge)
		{
			_writer.WriteLine("edge");
			_writer.WriteLine("[");
			_writer.WriteLine("source {0}", edge.SourceId);
			_writer.WriteLine("target {0}", edge.TargetId);
			
			_writer.WriteLine("graphics");
			_writer.WriteLine("[");
			
			if (edge.SmoothBends)
			{
				_writer.WriteLine("smoothBends 1");
			}
			
			if (edge.Style == EdgeStyle.Normal)
			{
				// it is the default
			}
			else
			{
				_writer.WriteLine("style \"{0}\"", yWorks.Constants.Encode(edge.Style));
			}
			
			_writer.WriteLine("fill \"{0}\"", yWorks.Constants.Encode(edge.Color));
			
			if (edge.Width > 1)
			{
				if (edge.Width > 5)
				{
					throw new NotSupportedException();
				}
				_writer.WriteLine("width {0}", edge.Width);
			}

			if (edge.TargetArrow == ArrowStyle.None)
			{
				// none is the default
			}
			else
			{
				_writer.WriteLine("targetArrow \"{0}\"", yWorks.Constants.Encode(edge.TargetArrow));
			}
			
			_writer.WriteLine("]");
			
			_writer.WriteLine("LabelGraphics");
			_writer.WriteLine("[");
			if (!String.IsNullOrEmpty(edge.Text))
			{
				_writer.WriteLine("text \"{0}\"", Encode(edge.Text));
				
				if (!edge.TextVisible)
				{
					_writer.WriteLine("visible 0");
				}
				
				if (!edge.BackgroundColor.IsEmpty)
				{
					_writer.WriteLine("fill \"{0}\"", yWorks.Constants.Encode(edge.BackgroundColor));
				}
				
				_writer.WriteLine("color \"{0}\"", yWorks.Constants.Encode(edge.TextColor));
				
				_writer.WriteLine("fontSize {0}", yWorks.Constants.EncodeFontSize(edge.Font));
				_writer.WriteLine("fontName \"{0}\"", edge.Font.Name);
				_writer.WriteLine("model \"three_center\"");
				// TODO: TUNE: label at source: writer.WriteLine("position \"scentr\"");
				_writer.WriteLine("position \"tcentr\"");
			}
			_writer.WriteLine("]");
			
			_writer.WriteLine("]");
		}
		
		public override void Write(Node node)
		{
			_writer.WriteLine("node");
			_writer.WriteLine("[");
			
			_writer.WriteLine("id {0}", node.Id);
			
			if (!String.IsNullOrEmpty(node.Text))
			{
				_writer.WriteLine("label \"{0}\"", Encode(node.Text));
			}
			_writer.WriteLine("graphics");
			_writer.WriteLine("[");
			
			if (!String.IsNullOrEmpty(node.Text))
			{
				System.Drawing.Size bounds = Canvas.MeasureString(node.Text, node.Font).ToSize();
				
				_writer.WriteLine("x {0}", 0);
				_writer.WriteLine("y {0}", 0);
				_writer.WriteLine("w {0}", bounds.Width);
				_writer.WriteLine("h {0}", bounds.Height);
			}
			
			_writer.WriteLine("type \"{0}\"", yWorks.Constants.Encode(node.Shape));
			
			if ((node.ShadowSize > 0) && !node.ShadowColor.IsEmpty)
			{
				_writer.WriteLine("dropShadowColor \"{0}\"", yWorks.Constants.Encode(node.ShadowColor));
				_writer.WriteLine("dropShadowOffsetX {0}", node.ShadowSize);
				_writer.WriteLine("dropShadowOffsetY {0}", node.ShadowSize);
			}

			if (!node.BorderColor.IsEmpty)
			{
				_writer.WriteLine("outline \"{0}\"", yWorks.Constants.Encode(node.BorderColor));
			}
			
			if (!node.BackgroundColor.IsEmpty)
			{
				_writer.WriteLine("fill \"{0}\"", yWorks.Constants.Encode(node.BackgroundColor));
			}
			
			_writer.WriteLine("]");
			
			_writer.WriteLine("LabelGraphics");
			_writer.WriteLine("[");
			if (!String.IsNullOrEmpty(node.Text))
			{
				_writer.WriteLine("text \"{0}\"", Encode(node.Text));
			}
			_writer.WriteLine("color \"{0}\"", yWorks.Constants.Encode(node.TextColor));
			_writer.WriteLine("fontSize {0}", yWorks.Constants.EncodeFontSize(node.Font));
			_writer.WriteLine("fontName \"{0}\"", node.Font.Name);
			_writer.WriteLine("anchor \"c\"");
			_writer.WriteLine("]");
			
			_writer.WriteLine("]");
		}
		
		public override void Close()
		{
			if (_writer != null)
			{
				_writer.WriteLine("]");
				
				_writer.Dispose();
				_writer = null;
			}
			base.Close();
		}
		#endregion
	}
}
