using System;

namespace Structure.Graph2D.IO
{
	public class ReaderBase
	{
		private IReaderCallback _visitor;
		
		private class DummyNode: INode
		{
		}
		
		private INode _dummyNode = new DummyNode();
		
		public ReaderBase(IReaderCallback visitor)
		{
			_visitor = visitor;
		}
		
		private class IDMap: System.Collections.Generic.Dictionary<string, int>
		{
			private int _seed = 0;
			
			public int Map(string id)
			{
				id = id.ToUpper();
				int result;
				if (!TryGetValue(id, out result))
				{
					_seed++;
					result = _seed;
					Add(id, result);
				}
				return result;
			}
		}

		private IDMap _ids = new IDMap();
		
		private class NodeMap: System.Collections.Generic.Dictionary<int, INode>
		{
			public bool Map(int id, out INode result)
			{
				if (TryGetValue(id, out result))
				{
					return true;
				}
				else
				{
					result = null;
					return false;
				}
			}
		}
		
		private NodeMap _nodes = new NodeMap();
		
		protected void Add(Node item)
		{
			Add(item, null);
		}
		
		protected INode Add(Node item, string id)
		{
			if (!String.IsNullOrEmpty(id))
			{
				item.Id = _ids.Map(id);
			}
			int nodeId = item.Id;
			INode node = _visitor.Add(item);
			if (node == null)
			{
				_nodes.Add(nodeId, _dummyNode);
			}
			else
			{
				_nodes.Add(nodeId, node);
			}
			return node;
		}
		
		private System.Collections.Generic.List<Edge> _edgeQueue = new System.Collections.Generic.List<Edge>();
		
		protected IEdge Add(Edge item)
		{
			return Add(item, null, null);
		}

		private bool Add(Edge item, out IEdge result)
		{
			INode sourceNode;
			INode targetNode;
			if (_nodes.Map(item.SourceId, out sourceNode) && _nodes.Map(item.TargetId, out targetNode))
			{
				if (sourceNode == _dummyNode)
				{
					sourceNode = null;
				}
				if (targetNode == _dummyNode)
				{
					targetNode = null;
				}
				result = _visitor.Add(item, sourceNode, targetNode);
				return true;
			}
			else
			{
				result = null;
				return false;
			}
		}
		
		protected IEdge Add(Edge item, string sourceId, string targetId)
		{
			if (!String.IsNullOrEmpty(sourceId))
			{
				item.SourceId = _ids.Map(sourceId);
			}
			if (!String.IsNullOrEmpty(targetId))
			{
				item.TargetId = _ids.Map(targetId);
			}
			IEdge result;
			if (!Add(item, out result))
			{
				// nodes are not known yet. delay the link
				_edgeQueue.Add(item);
			}
			return result;
		}
		
		public virtual void Close()
		{
			foreach(Edge item in _edgeQueue)
			{
				IEdge result;
				Add(item, out result);
			}
			_edgeQueue.Clear();
		}
	}
}
