using System;

namespace System.Configuration
{
	public class CommandLine
	{
		private class Reader
		{
			public Reader()
			{
				string[] commandLine = System.Environment.GetCommandLineArgs();
				string[] args = new string[commandLine.Length - 1];
				for(int i = 0; i < args.Length; i++)
				{
					args[i] = commandLine[1 + i];
				}
				Initialize(args);
			}

			public Reader(string[] args)
			{
				Initialize(args);
			}

			public int Args = 0;
			private System.Collections.Specialized.StringDictionary Opts = new System.Collections.Specialized.StringDictionary();

			private void Initialize(string[] args)
			{
				int i = 0;
				while (i < args.Length)
				{
					string Opt = args[i].Trim();
					if (Opt == "")
					{
						i++;
						continue;
					}
					if (Opt.IndexOfAny(new char[]{'/', '-'}, 0, 1) == 0)
					{
						// generic option syntax:
						// <optSwitch><optName>[:<optValue>]
						// <optSwitch><optName>[=<optValue>]
						// <optSwitch><optName>[<boolFlag>]
						Opt = Opt.Remove(0, 1);
						if (Opt == "")
						{
							i++;
							continue;
						}
						int j = Opt.IndexOfAny(new char[]{':', '='});
						if (j < 0)
						{
							switch(Opt[Opt.Length - 1])
							{
								case '+':
								case '-':
									Opts.Add(Opt.Substring(0, Opt.Length -1).ToUpper(), Opt.Substring(Opt.Length - 1, 1));
									break;

								default:
									Opts.Add(Opt.ToUpper(), "");
									break;
							}
						}
						else
						{
							Opts.Add(Opt.Substring(0, j).ToUpper(), Opt.Substring(j + 1));
						}
						i++;
					}
					else
					{
						Opts.Add(Args.ToString(), Opt);
						Args++;
						i++;
					}
				}
			}

			public object GetValue(int index, object defaultValue)
			{
				return GetValue(index.ToString(), defaultValue);
			}

			public object GetValue(string index, object defaultValue)
			{
				string key = index.ToUpper();
				if (Opts.ContainsKey(key))
				{
					return Opts[key];
				}
				else
				{
					return defaultValue;
				}
			}
		}

		private static Reader pReader = null;
		private static Reader reader
		{
			get
			{
				if (pReader == null)
				{
					pReader = new Reader();
				}
				return pReader;
			}
		}

		public static string[] Args
		{
			get
			{
				string[] result = new string[reader.Args];
				for(int i =0; i < result.Length; i++)
				{
					result[i] = (string)reader.GetValue(i, null);
				}
				return result;
			}
		}

		public static string Command
		{
			get
			{
				string result = null;
				foreach(string word in Args)
				{
					if (!String.IsNullOrEmpty(result))
					{
						result += " ";
					}
					result += word;
				}
				return result;
			}
		}

		public static string GetString(string key, string defaultValue) 
		{
			object result = reader.GetValue(key, defaultValue);
			return (result == null) ? null : System.Convert.ToString(result);
		}

		public static string GetString(string key)
		{
			return GetString(key, null);
		}

		public static bool GetBoolean(string key, bool defaultValue)
		{
			string value = (string)reader.GetValue(key, null);
			switch(value)
			{
				case "+":
				case "":
					return true;

				case "-":
					return false;

				default:
					return defaultValue;
			}
		}

		public static bool GetBoolean(string key)
		{
			return GetBoolean(key, false);
		}

		public static float GetSingle(string key, float defaultValue) 
		{
			return System.Convert.ToSingle(reader.GetValue(key, defaultValue), System.Globalization.NumberFormatInfo.InvariantInfo);
		}

		public static float GetSingle(string key)
		{
			return GetSingle(key, 0);
		}

		public static int GetInt32(string key, int defaultValue) 
		{
			return System.Convert.ToInt32(reader.GetValue(key, defaultValue));
		}

		public static int GetInt32(string key)
		{
			return GetInt32(key, 0);
		}
	}
}
