using System;

namespace System.IO
{
	public class PlainFileFilter: System.Text.StringFilter
	{
		public PlainFileFilter(): base(false)
		{
		}
		
		public PlainFileFilter(string masks): base(masks, false)
		{
		}
		
		protected override string NormalizedText(string text)
		{
			string result = System.IO.Path.GetFileName(text);
			if (String.IsNullOrEmpty(System.IO.Path.GetExtension(result)))
			{
				return result + ".";
			}
			else
			{
				return result;
			}
		}
		
		public override bool AcceptsAll()
		{
			if (base.AcceptsAll())
			{
				return true;
			}
			if (ExcludeMasks.Length > 0)
			{
				return false;
			}
			foreach (string mask in IncludeMasks)
			{
				if (mask == "*")
				{
					return true;
				}
				if (mask == "*.*")
				{
					return true;
				}
			}
			return false;
		}
	}
	
	public class PlainFolderFilter: PlainFileFilter
	{
		public PlainFolderFilter()
		{
		}
		
		public PlainFolderFilter(string masks): base(masks)
		{
		}
		
		protected override string NormalizedText(string text)
		{
			return text;
		}
	}
	
	public class FileFilter: System.Text.StringFilter
	{
		public FileFilter(): base(false)
		{
		}

		public FileFilter(string masks): base(false)
		{
			base.Add(masks);
		}
		
		private PlainFileFilter _fileFilter = new PlainFileFilter();
		private PlainFolderFilter _folderFilter = new PlainFolderFilter();
		
		private bool IsFolderMask(string mask)
		{
			return mask.IndexOfAny(new char[]{':', '\\'}) >= 0;
		}
		
		protected override void AddIncludeMask(string mask)
		{
			if (IsFolderMask(mask))
			{
				_folderFilter.Include(mask);
			}
			else
			{
				_fileFilter.Include(mask);
			}
		}
		
		protected override void AddExcludeMask(string mask)
		{
			if (IsFolderMask(mask))
			{
				_folderFilter.Exclude(mask);
			}
			else
			{
				_fileFilter.Exclude(mask);
			}
		}
		
		public override bool Accepts(string text)
		{
			return _folderFilter.Accepts(text) && _fileFilter.Accepts(text);
		}
		
		public override bool AcceptsAll()
		{
			return _folderFilter.AcceptsAll() && _fileFilter.AcceptsAll();
		}
	}
}
