using System;

namespace System.IO
{
	public class PathBuilder
	{
		public PathBuilder()
		{
		}

		public PathBuilder(string path)
		{
			FullPath = path;
		}

		public System.Collections.Generic.List<string> Items = new System.Collections.Generic.List<string>();

		public static string GetRelativePath(string fromPath, string toPath)
		{
			PathBuilder fromPathSplit = new PathBuilder(fromPath);
			return fromPathSplit.GetRelativePath(toPath);
		}

		public string GetRelativePath(string toPath)
		{
			PathBuilder toPathSplit = new PathBuilder(toPath);

			int countOfSharedComponents = 0;
			while ((countOfSharedComponents < Items.Count) &&
				   (countOfSharedComponents < toPathSplit.Items.Count) &&
				   (String.Compare(Items[countOfSharedComponents], toPathSplit.Items[countOfSharedComponents], true) == 0))
			{
				countOfSharedComponents++;
			}

			PathBuilder result = new PathBuilder();
			for(int i = Items.Count - countOfSharedComponents; i > 0; i--)
			{
				result.Items.Add("..");
			}
			for(int i = countOfSharedComponents; i < toPathSplit.Items.Count; i++)
			{
				result.Items.Add(toPathSplit.Items[i]);
			}
			return result.FullPath;
		}

		public string FullPath
		{
			get
			{
				string result = null;
				foreach(string item in Items)
				{
					if (result == null)
					{
						result = item;
					}
					else
					{
						// TODO: use path separator from System.IO.Path
						result += "\\" + item;
					}
				}
				return result;
			}
			set
			{
				Items.Clear();
				if (!String.IsNullOrEmpty(value))
				{
					string buffer = System.IO.Path.GetFullPath(value).Replace("/", "\\").Replace("\\\\", "\\");
					foreach(string item in buffer.Split('\\'))
					{
						Items.Add(item);
					}
				}
			}
		}
	}
}
