using System;

namespace System.Windows.Forms
{
	public class KeyHandler
	{
		private object _control;
		
		public KeyHandler(System.Windows.Forms.Control owner)
		{
			Initialize(owner, null);
		}
		
		public KeyHandler(System.Windows.Forms.Control owner, KeyEventHandler keyDownUp)
		{
			Initialize(owner, keyDownUp);
		}

		private void Initialize(System.Windows.Forms.Control owner, KeyEventHandler keyDownUp)
		{
			_control = owner;
			owner.KeyDown += this.KeyDownHandler;
			owner.KeyUp += this.KeyUpHandler;
			if (keyDownUp != null)
			{
				KeyDownUp += keyDownUp;
			}
		}
		
		public KeyHandler(System.Windows.Forms.ToolStripControlHost owner)
		{
			Initialize(owner, null);
		}
		
		public KeyHandler(System.Windows.Forms.ToolStripControlHost owner, KeyEventHandler keyDownUp)
		{
			Initialize(owner, keyDownUp);
		}
		
		private void Initialize(System.Windows.Forms.ToolStripControlHost owner, KeyEventHandler keyDownUp)
		{
			_control = owner;
			owner.KeyDown += this.KeyDownHandler;
			owner.KeyUp += this.KeyUpHandler;
			owner.Control.PreviewKeyDown += this.KeyDownPreviewHandler;
			if (keyDownUp != null)
			{
				KeyDownUp += keyDownUp;
			}
		}
		
		private KeyEventArgs _key = null;
		
		private void KeyDownPreviewHandler(object sender, PreviewKeyDownEventArgs e)
		{
			// special key that doesn't go to the key down
			// see https://forums.microsoft.com/Forums/ShowPost.aspx?PostID=142048&SiteID=1
			// this fixes bug when Ctrl+Insert activates menustrip instead of being passed to the toolstrip item
			if ((e.KeyCode == Keys.Insert) && e.Control)
			{
				e.IsInputKey = true;
			}
		}
		
		private void KeyDownHandler(object sender, KeyEventArgs e)
		{
			_key = e;
		}
		
		private void KeyUpHandler(object sender, KeyEventArgs e)
		{
			if ((e.KeyCode == Keys.Tab) && (e.Modifiers == Keys.None))
			{
				// special key that doesn't go to the key down
				e.Handled = true;
				ProcessKey(e);
				_key = null;
				return;
			}
			
			if (_key == null)
			{
				return;
			}
			
			if ((_key.KeyCode == e.KeyCode) && (_key.KeyData == e.KeyData) && (_key.KeyValue == e.KeyValue) && (_key.Modifiers == e.Modifiers))
			{
				// key up corresponds with key down
				e.Handled = true;
				ProcessKey(_key);
			}
			
			_key = null;
		}
		
		public KeyEventHandler KeyDownUp = null;
		
		private void ProcessKey(KeyEventArgs e)
		{
			if (KeyDownUp != null)
			{
				KeyDownUp(_control, e);
			}
		}
	}
}
