using System;
using System.Runtime.InteropServices;

namespace System.Windows.Forms
{
	public class SendKeysHelper
	{
		[DllImport("user32.dll", EntryPoint="SystemParametersInfo")]
		public static extern bool SystemParametersInfo(uint uiAction, uint uiParam, uint pvParam, uint fWinIni);
		
		[DllImport("user32.dll", EntryPoint="SetForegroundWindow")]
		public static extern bool SetForegroundWindow(IntPtr hWnd);
		
		[DllImport("User32.dll", EntryPoint="ShowWindowAsync")]
		private static extern bool ShowWindowAsync(IntPtr hWnd, int cmdShow);
		private const int WS_SHOWNORMAL = 1;
		
		[DllImport( "User32.dll", EntryPoint="FindWindow", CharSet=CharSet.Auto)]
		public static extern IntPtr FindWindow(String className, String caption);

		public SendKeysHelper()
		{
		}
		
		public static bool SelectWindow(IntPtr handle)
		{
			if (handle == IntPtr.Zero)
			{
				return false;
			}
			SystemParametersInfo( (uint) 0x2001, 0, 0, 0x0002 | 0x0001);
			ShowWindowAsync(handle, WS_SHOWNORMAL);
			bool result = SetForegroundWindow(handle);
			SystemParametersInfo( (uint) 0x2001, 200000, 200000, 0x0002 | 0x0001);
			return result;
		}
		
		public static bool SelectWindow(string windowName)
		{
			return SelectWindow(windowName, 0);
		}
		
		public static bool SelectWindow(string windowName, Int32 timeout)
		{
			return SelectWindow(FindWindow(windowName, timeout));
		}
		
		private static bool Wait(Int32 timeout)
		{
			return (timeout == System.Threading.Timeout.Infinite) || (timeout > 0);
		}
		
		private static Int32 WaitTime(ref Int32 timeout, Int32 slice)
		{
			if (timeout == System.Threading.Timeout.Infinite)
			{
				return slice;
			}
			if (slice >= timeout)
			{
				slice = timeout;
				timeout = 0;
				return slice;
			}
			timeout = timeout - slice;
			return slice;
		}
		
  		// Find window by Caption, and wait 1/2 a second and then try again.
        public static IntPtr FindWindow(string windowName, Int32 timeout)
        {
            IntPtr hWnd = FindWindow(null, windowName);
            while (Wait(timeout) && (hWnd == IntPtr.Zero))
            {
            	System.Threading.Thread.Sleep(WaitTime(ref timeout, 500));
                hWnd = FindWindow(null, windowName);
            }

            return hWnd;
        }
        
        private static string GetExeFile(System.Diagnostics.Process p)
        {
        	try
        	{
        		System.Diagnostics.ProcessModule module = p.MainModule;
        		if (module == null)
        		{
        			return null;
        		}
        		return System.IO.Path.GetFileNameWithoutExtension(module.FileName);
        	}
        	catch
        	{
        		return null;
        	}
        }
        
        private static IntPtr FindProcessWindow(string processName)
        {
        	processName = System.IO.Path.GetFileNameWithoutExtension(processName);
			foreach(System.Diagnostics.Process p in System.Diagnostics.Process.GetProcesses())
			{
				if (String.Compare(processName, GetExeFile(p), true) == 0)
				{
					return p.MainWindowHandle;
				}
			}
			return System.IntPtr.Zero;        	
        }
        
        // Find main window by process name. Optionally wait 1/2 a second and try again
        public static IntPtr FindProcessWindow(string processName, Int32 timeout)
        {
            IntPtr hWnd = FindProcessWindow(processName);
            while (Wait(timeout) && (hWnd == IntPtr.Zero))
            {
            	System.Threading.Thread.Sleep(WaitTime(ref timeout, 500));
                hWnd = FindProcessWindow(processName);
            }

            return hWnd;
        }
        
		public static bool Send(string process, string keys, Int32 timeout)
		{
			IntPtr handle = System.Windows.Forms.SendKeysHelper.FindProcessWindow(process, timeout);
			if (System.Windows.Forms.SendKeysHelper.SelectWindow(handle))
			{
				if (keys == "{CLOSE}")
				{
					foreach(System.Diagnostics.Process p in System.Diagnostics.Process.GetProcesses())
					{
						if (p.MainWindowHandle == handle)
						{
							return p.CloseMainWindow();
						}
					}
					return false;
				}
				else
				{
					System.Windows.Forms.SendKeys.SendWait(keys);
					return true;
				}
			}
			else
			{
				return false;
			}
		}
	}
}
