using System;

namespace System.Xml
{
	public class XmlNodeVisitor
	{
		private XmlNode _rootNode;
		protected XmlNode RootNode
		{
			get
			{
				return _rootNode;
			}
		}
		
		protected string GetPath(XmlNode node)
		{
			if ((node == null) || (node == _rootNode))
			{
				return null;
			}
			else
			{
				string result = GetPath(node.ParentNode);
				if (String.IsNullOrEmpty(result))
				{
					return node.Name;
				}
				else
				{
					return result + "/" + node.Name;
				}
			}
		}
		
		protected string GetPath(XmlNode node, XmlAttribute attribute)
		{
			return GetPath(node) + "/@" + attribute.Name;
		}
		
		public virtual void BeginVisit(XmlNode node)
		{
		}
		
		public virtual void EndVisit(XmlNode node)
		{
		}
		
		public virtual void Visit(XmlNode node, XmlAttribute attribute)
		{
		}
		
		private bool InternalWalk(XmlNode node)
		{
			XmlNode oldParent = node.ParentNode;
			BeginVisit(node);
			if (node.ParentNode != oldParent)
			{
				// node was moved or removed
				return false;
			}
			if (node.Attributes != null)
			{
				foreach(XmlAttribute attribute in node.Attributes)
				{
					Visit(node, attribute);
				}
			}
			for(int i = 0; i < node.ChildNodes.Count;)
			{
				if (InternalWalk(node.ChildNodes[i]))
				{
					i++;
				}
			}
			EndVisit(node);
			return true;
		}
		
		public void Walk(XmlNode node)
		{
			_rootNode = node;
			InternalWalk(node);
		}
		
		public void Walk(XmlNodeList nodes)
		{
			for(int i = 0; i < nodes.Count; i++)
			{
				Walk(nodes[i]);
			}
		}
	}
}
